@@ -33,13 +33,18 @@
     print(mispelled) # ["bill", "tv", "borken"]
 
     pspell = phunspell.Phunspell("cs_CZ")
-
 """
 
 import os
 import string
+import pickle
+import tempfile
+
+# import functools
 from spylls.hunspell import Dictionary
 
+TEMPDIR = tempfile.gettempdir()
+
 DICTIONARIES = {
     # lang-loc # dir   # language
     "af_ZA": ["af_ZA", "Afrikaans"],
@@ -136,30 +141,73 @@ DICTIONARIES = {
     # "hyph_zu_ZA" : ["zu_ZA",    "LANG"],
 }
 
+# memoize
+DICTIONARIES_LOADED = {}
+
 
 class PhunspellError(Exception):
     def __init__(self, message):
         Exception.__init__(self, "%s" % (message))
 
+class PhunspellObjectStore:
+    """Create local object stores of all dictionaries
+    meant to only be run once.
+
+    NOTE: Will need to be rerun anytime a dictionary is updated
+    """
+
+    def __init__(self, path):
+        """NOTE: meant to be run once to
+                 create/store local pickled dictionary objects
+        """
+        try:
+            pspell = Phunspell(object_storage=path)
+            pspell.dictionary_loader(pspell.dictionaries_all())
+        except (
+            FileNotFoundError,
+            KeyError,
+            TypeError,
+            ValueError,
+            PhunspellError,
+        ) as error:
+            raise PhunspellError(
+                "phunspell object store, dictionary not found {}".format(error)
+            )
 
 class Phunspell:
     """pure Python spell checker, utilizing Spylls a port of Hunspell"""
 
-    def __init__(self, loc_lang):
+    def __init__(self,
+                 loc_lang="en_US",
+                 loc_list=[],
+                 object_storage=None):
         """Load passed dictionary (loc_lang) on init
         dictionary must be of the form Locale_Langage
         e.g. "en_US"
 
+        Can specify a local directory path to dictionary objects
+
+        params:
+            loc_list : ["en_US", "de_AT", "de_DE", "el_GR", etc]
+            object_storage : "/home/dwright/python/phunspell/pickled_data/"
+
         See README for all supported languages
         """
         try:
-            self.dict_dirpath(loc_lang)
-            # print(self.dict_path)
+            self.object_storage = object_storage
 
-            # use local dictionaries only
-            self.dictionary = Dictionary.from_files(self.dict_path)
-            # pass directly to spylls (could do for en_US)
-            # self.dictionary = Dictionary.from_files(loc_lang)
+            if len(loc_list) > 0:
+                # if locales passed, load specified dictionaries on init
+                self.dictionary_loader(loc_list)
+            else:
+                # just load single loc dictionary on init
+                self.dict_dirpath(loc_lang)
+
+                # use local dictionaries only
+                self.dictionary = Dictionary.from_files(self.dict_path)
+
+                # pass directly to spylls (could do for en_US)
+                # self.dictionary = Dictionary.from_files(loc_lang)
         except (
             FileNotFoundError,
             KeyError,
@@ -171,11 +219,80 @@ class Phunspell:
                 "phunspell, dictionary not found {}".format(error)
             )
 
+    def dictionaries_all(self):
+        """Load 'all' dictionaries"""
+        return [n for n in DICTIONARIES.keys() if n.find('_') != -1]
+
+    # TODO: debug
+    # @functools.lru_cache
+    def dictionary_load(self, loc):
+        """load stored dictionary object for locale"""
+        try:
+            if loc in DICTIONARIES_LOADED:
+                self.dictionary = DICTIONARIES_LOADED[loc]
+                return
+
+            if self.object_storage:
+                filepath = os.path.join(self.object_storage, loc)
+            else:
+                filepath = os.path.join(TEMPDIR, loc)
+
+            pfile = open(filepath, 'rb')
+            stored_dic = pickle.load(pfile)
+            self.dictionary = stored_dic
+            pfile.close()
+
+            # memoize
+            DICTIONARIES_LOADED[loc] = stored_dic
+        except (TypeError, OSError) as error:
+            raise PhunspellError(f'Cannot load dictionary: {error}')
+
+    def dictionary_store(self, loc):
+        """iterate locale dump dictionary to object
+        pickle the dictionary
+        """
+        try:
+            if self.object_storage:
+                filepath = os.path.join(self.object_storage, loc)
+            else:
+                filepath = os.path.join(TEMPDIR, loc)
+
+            # if os.path.exists(filepath):
+            self.dict_dirpath(loc)
+            pfile = open(filepath, 'wb')
+            data = Dictionary.from_files(self.dict_path)
+            pickle.dump(data, pfile)
+            pfile.close()
+            # else:
+            #   raise PhunspellError(f'Cant create dictionay path {filepath}')
+        except (TypeError, OSError) as error:
+            raise PhunspellError(f'Cannot write file: {filepath} {error}')
+
+    def dictionary_loader(self, loc_list):
+        """iterate locale list load dictionary for locale
+
+        meant to load dictionaries on init, not return them
+        from this call
+        """
+        for loc in loc_list:
+            # XXX skip for now
+            # re.error: bad character range ű-ø at position 12
+            if loc in ["hu_HU"]:
+                continue
+
+            self.dictionary_store(loc)
+
     def find_dict_dirpath(self, dictdir, loc_lang):
-        # expected "underscore" format "en_US"
+        """find directory for dictionary `loc_lang`
+
+        if found sets:
+            self.dict_path = dictionary directory
+            self.loc_lang = language locale
+        """
+        # expecting "underscore" format. e.g. "en_US"
         if os.path.isdir(dictdir):
             aff_file = os.path.join(dictdir, "{}.aff".format(loc_lang))
-            # expected "underscore" format "en_US.aff"
+            # expecting "underscore" format. e.g. "en_US.aff"
             if os.path.exists(aff_file):
                 self.dict_path = os.path.join(dictdir, loc_lang)
                 self.loc_lang = loc_lang
@@ -193,6 +310,7 @@ class Phunspell:
         raise PhunspellError("phunspell, dictionary path not found")
 
     def dict_dirpath(self, loc_lang):
+        """find directory to dictionary for locale"""
         try:
             dirpath = os.path.dirname(os.path.realpath(__file__))
             dictdir = os.path.join(
@@ -238,19 +356,22 @@ class Phunspell:
         except (FileNotFoundError, TypeError, ValueError) as error:
             raise PhunspellError("phunspell, to list failed {}".format(error))
 
-    def lookup(self, word):
+    def lookup(self, word, locs=[]):
         """takes word (string)
         removes beginning and trailing whitespace
         returns true if found in dictionary, false otherwise
         """
         try:
+            if len(locs) > 0:
+                self.dictionary_load(locs)
+
             return self.dictionary.lookup(word.strip())
         except ValueError as error:
             raise PhunspellError(
                 "phunspell, dictionary lookup failed {}".format(error)
             )
 
-    def lookup_list(self, wordlist):
+    def lookup_list(self, wordlist, locs=[]):
         """takes list of words, returns list words not found
         in dictionary
 
@@ -258,6 +379,9 @@ class Phunspell:
         Out: ["programing"]
         """
         try:
+            if len(locs) > 0:
+                self.dictionary_load(locs)
+
             flagged = []
             for word in wordlist:
                 if not self.dictionary.lookup(word.strip()):
@@ -268,13 +392,16 @@ class Phunspell:
                 "phunspell, dictionary lookup_list failed {}".format(error)
             )
 
-    def suggest(self, word):
+    def suggest(self, word, locs=[]):
         """takes word (string)
         removes beginning and trailing whitespace
         returns suggested spelling if not found in dictionary
         returns nothing if found in dictionary
         """
         try:
+            if len(locs) > 0:
+                self.dictionary_load(locs)
+
             return self.dictionary.suggest(word.strip())
         except ValueError as error:
             raise PhunspellError(
@@ -291,11 +418,56 @@ class Phunspell:
 
 
 if __name__ == "__main__":
-    pspell = Phunspell("en_US")
+    # create pickled dictionaries for all dictionaries
+    # import tempfile
+    # TEMPDIR = tempfile.gettempdir()
+    #storage_path = "/var/folders/vd/n_l9hxb57msdcc_33myhkqmw0000gn/T"
+    storage_path = "/tmp/foo"
+
+    # run once:
+    # pspell = PhunspellObjectStore(path=storage_path)
+    # import sys
+    # sys.exit()
+
+    dicts_words = {
+        "af_ZA": "voortgewoed",
+        "an_ES": "vengar",
+        "be_BY": "ідалапаклонніцкі",
+        "bg_BG": "удържехме",
+        "br_FR": "c'huñvderioù",
+        "de_DE": "schilffrei",
+        "en_GB": "indict",
+        "es_MX": "pianista",
+        "fr_FR": "zoomorphe",
+    }
+    # use cache if already seen
+    dicts_words_cached = {
+        "an_ES": "vengar",
+        "be_BY": "ідалапаклонніцкі",
+        "bg_BG": "удържехме",
+    }
+
+    # 16.41s user 0.48s system 99% cpu 16.986 total
+    pspell = Phunspell(object_storage=storage_path)
+
+    for loc in dicts_words.keys():
+        # 36.08s user 0.65s system 99% cpu 36.788 total
+        # pspell = Phunspell(loc)
+        print(pspell.lookup(dicts_words[loc], locs=loc))
+
+    for loc in dicts_words_cached.keys():
+        # 36.08s user 0.65s system 99% cpu 36.788 total
+        # pspell = Phunspell(loc)
+        print(pspell.lookup(dicts_words[loc], locs=loc))
+
+    # import sys
+    # sys.exit()
+
+    pspell = Phunspell(loc_lang="en_US")
     print(pspell.lookup_list("Wonder Woman 1984"))
     print(pspell.lookup_list(pspell.to_list("Wonder Woman 1984")))
 
-    pspell = Phunspell("en_US")
+    pspell = Phunspell()  # default "en_US"
     # pspell = Phunspell("af_ZA")
 
     print(pspell.lookup("phunspell"))  # False
